sample_prior <- function(n,hyper.parameters) { 
  #This function will draw n (integer) random sample from the 
  #joint distribution of the parameters.  hyper.parameters contains the values
  #of the hyperparameters for the prior  distribution
  
  require(MASS)
  
  ##Object to store the draws from the prior distribution
  ##Columns 1-2 are beta and gamma, columns 3-4 are sigma.b and sigma
  draws <-matrix(0,nrow = n, ncol = 4)
  
  ##Extract parameters from hyper.parameters
  mu.theta = c(hyper.parameters[1],hyper.parameters[3])
  sigma.theta = matrix(c(hyper.parameters[2],hyper.parameters[5],
                         hyper.parameters[5], hyper.parameters[4]),nrow=2)
  
  ##Simulate from the respective priors
  draws[,1:2] = mvrnorm(n = n, mu = mu.theta, Sigma = sigma.theta)
  draws[,3] <- rlnorm(n,meanlog = hyper.parameters[6], sdlog = hyper.parameters[7])
  draws[,4] <- rlnorm(n,meanlog =hyper.parameters[8], sdlog = hyper.parameters[9])
  
  return(draws)
}
adt.fisher.mat <- function(pv,time.vec,accel.level){
  ##Calculate the fisher information matrix
  #accel.level is the accelerated level for unit i
  #i determines which time sampling to use
  #First will get the covar. matrix.
  #Beginning with acquiring Z, the sampling matrix,
  #phi.b is the covar matrix for vector b
  #sig.b is the covariance matrix of y.i
  #XX corresponds to \mathbf{x} in the RMADT paper, Section planning specification
  
  n.pv <- length(pv)     #Number of parameters
  f.pv <- n.pv - 2       #Number of fixed effects
  m.i = length(time.vec) #Number of measurements
  
  XX <- cbind(time.vec,time.vec*accel.level)  #Model matrix for fixed effects
  Z = time.vec
  phi.b = pv[3]^2 
  
  sig.b = phi.b*Z%*%t(Z)+pv[4]^2*diag(1,m.i,m.i)		
  
  #Now will create the elements of the matrix M.i
  sig.i1 = 2*pv[3]*Z%*%t(Z)         ##sigma b
  sig.i2 = 2*pv[4]*diag(1,m.i,m.i)  ##sigma
  
  M.i = M.mat(sig.i1,sig.i2,sig.b)
  
  #Now will create the ith element of the information matrix sum
  X = t(XX)%*%solve(sig.b)%*%XX
  I.i = matrix(0,n.pv,n.pv)
  I.i[1:f.pv,1:f.pv] = X
  I.i[(f.pv+1):n.pv,(f.pv+1):n.pv] = M.i
  return(I.i)
}
#------------------------------------------------------------------------------------------
#--------------------------------M matrix in Fisher function-------------------------------------------
M.mat = function(w,x,sig.b){
  #w,x are the derivative matrices, sig.b is the covariance matrix	
  base.mat = matrix(0,2,2)
  
  #Array used for loops
  mat.array = array(c(w,x),c(nrow(w),ncol(w),2))
  
  for(i in 1:2){
    for(j in 1:2){
      base.mat[i,j] = .5*sum(diag(solve(sig.b)%*%mat.array[,,i]%*%solve(sig.b)%*%mat.array[,,j]))
    }
  }
  
  return(base.mat)
}

my.gradient <- function(f.fun, theta, function.element = 1., delta = 9.9999999999999995e-007 * theta, ...)
{
  #Calculate numerical derivatives for the function f.fun at theta
  delta.eq.zero <- delta == 0.
  if(any(delta.eq.zero))
    delta[delta.eq.zero] <- 1.0000000000000001e-005
  gradient <- rep(NA, length(theta))
  for(i in 1.:length(theta)) {
    deltai <- rep(0., length(theta))
    deltai[i] <- delta[i]
    gradient[i] <- (f.fun(theta + deltai, ...)[function.element] - f.fun(theta - deltai, ...)[
      function.element])/(2. * delta[i])
  }
  names(gradient) <- names(theta)
  return(gradient)
}

integrand <- function(draws,eta,times,p,Deg,S,accel,N,less.than = F){
  #This function will calculate the integrand for draws of parameters
  #and for a given test plan. eta[,1] is pi_L and eta[,2] is \xi_L
  #draws, times,p, Deg, S, accel, and N are for the posterior draws, the inspection times,,
  ##quantile, degradation threshold,
  ##prior precision matrix, accelerating level, and sample size
  
  pv <- get.ADT.RMD.pv(beta = draws[1], gamma = draws[2],
                       sigma.beta = draws[3], sigma.measurement = draws[4])
  
  cc <- my.gradient(f.fun = adt.life.quantile, theta = pv, p = p, Deg = Deg,
                    accel = accel,less.than = less.than)
  
  eta.u <- unique(eta)
  t.id <- which(eta == eta.u[1])
  f.I <- (length(t.id)/length(eta))*adt.fisher.mat(pv,times,eta.u[1])
  for(i in 2:length(eta.u)){
    t.id <- which(eta == eta.u[i])
    
    f.I <- f.I + (length(t.id)/length(eta))*adt.fisher.mat(pv,times,eta.u[i])
  }
  
  x <- t(cc) %*% solve(S + N*f.I) %*% cc
  return(x)
}

exp.utility <- function(draws,eta,S,times,p,Deg,accel,N){
  #This function will calculate the expected utility
  #for a given test plan eta. eta[,1] is pi_L and eta[,2] is \xi_L
  #draws, times,p, Deg, S, accel, and N are for the posterior draws, the inspection times,,
  ##quantile, degradation threshold,
  ##prior precision matrix, accelerating level, and sample size
  
  vals <- apply(draws,1,integrand,eta = eta,times = times,p = p,
                Deg = Deg,S = S,accel = accel,N=N)
  
  return(-mean(vals))
}

frechet.der <- function(draws,eta.c,eta,accel,S,times,p,Deg,N = N,less.than = F){
  #This function will calculate the bayesian frechet (directional) derivative
  ##for design eta and pointing in the direction eta.c (Singular design)
  #draws, times,p, Deg, S, accel, and N are for the posterior draws, the inspection times,,
  ##quantile, degradation threshold,
  ##prior precision matrix, accelerating level, and sample size 
  vals <- apply(draws,1,integrand.der,xi.l = eta.c,eta = eta,S = S,
                times = times,N=N,p = p,Deg = Deg,accel = accel,less.than = less.than)
  return(vals)
}

integrand.der <- function(draws,eta,xi.l,accel,S,times,p,Deg,N=N, less.than = F){
  #This function will calculate the integrand involved with the frechet derivative at
  ##design eta
  #draws, times,p, Deg, S, accel, and N are for the posterior draws, the inspection times,,
  ##quantile, degradation threshold,
  ##prior precision matrix, accelerating level, and sample size
  pv <- get.ADT.RMD.pv(beta = draws[1], gamma = draws[2],
                       sigma.beta = draws[3], sigma.measurement = draws[4])
  
  cc <- my.gradient(f.fun = adt.life.quantile, theta = pv, p = p, Deg = Deg,
                    accel = accel,less.than = less.than)
  eta.u <- unique(eta)
  t.id <- which(eta == eta.u[1])
  f.I <- (length(t.id)/length(eta))*adt.fisher.mat(pv,times,eta.u[1])
  for(i in 2:length(eta.u)){
    t.id <- which(eta == eta.u[i])
    
    f.I <- f.I + (length(t.id)/length(eta))*adt.fisher.mat(pv,times,eta.u[i])
  }
  
  f.I2 <- adt.fisher.mat(pv = pv,time.vec = times,accel.level = xi.l)
  v1 <- t(cc) %*% solve(S + N*f.I)%*% (S + N*f.I2) %*% solve(S + N*f.I)%*% cc
  return(v1)
  
}

prior.draws <- function(pv,n){
  #This function will draw a random sample from the 
  #joint distribution of the parameters.  pvs is the values
  #of the hyperparameters for the prior  distribution
  
  draws <-matrix(0,nrow = n, ncol = 4)
  
  draws[,1] <- rnorm(n,mean = pv[1], sd = pv[2])
  draws[,2] <- rlnorm(n,meanlog = pv[3], sdlog = pv[4])
  draws[,3] <- rlnorm(n,meanlog = pv[5], sdlog = pv[6])
  draws[,4] <- rlnorm(n,meanlog = pv[7], sdlog = pv[8])
  
  return(draws)
}


adt.life.quantile <- function(pv,p,Deg,accel,less.than = F){
  
  #This function calculates the quantile of the lifetime distribution of an
  #accelerated linear repeated measures degradation model
  #pv is the parameter values
  #p is the probability associated with the desired quantile
  #Deg is the point of degradation for which a failure will occur
  #accel is the level of the accelerating variable
  
  if(less.than == F){
    k <- qnorm(1-p)
  }
  if(less.than == T){
    k <- qnorm(p)
  }
  
  ##First just defining some constants
  h <- Deg
  r <- pv[1] + pv[2]*accel
  
  ##now defining the constants in the quadratic equation a*t^2+b*t+c
  t.p <- (h/(pv[3]*k+r))^2
  
  return(t.p)
}
get.ADT.RMD.pv <- function(beta, gamma, sigma.beta, sigma.measurement)
{
  true.parameters <- c(beta = beta, gamma = gamma,
                       sigma.beta = sigma.beta, sigma.measurement = sigma.measurement)
  oldClass(true.parameters) <- "RMD.pv"
  return(true.parameters)
}

comp.integrand <- function(draws,pis,x.low,x.mid,x.max,times,p,Deg,S,accel,N = N,less.than = F){
  #This function will calculate the integrand for compromise plans
  ###pis is the allocation,x.low, x.mid, and x.max is 
  ##the compromise points for low, middle, and max levels
  #draws, times,p, Deg, S, accel, and N are for the posterior draws, the inspection times,,
  ##quantile, degradation threshold,
  ##prior precision matrix, accelerating level, and sample size
  
  pi.l <- pis[1]
  pi.m <- pis[2]
  
  pv <- get.ADT.RMD.pv(beta = draws[1], gamma = draws[2],
                       sigma.beta = draws[3], sigma.measurement = draws[4])
  
  cc <- my.gradient(f.fun = adt.life.quantile, theta = pv, p = p, Deg = Deg, 
                    accel = accel,less.than = less.than)
  
  f.I <- pi.l*adt.fisher.mat(pv,times,x.low) + pi.m*adt.fisher.mat(pv,times,x.mid) +
    (1-pi.l-pi.m)*adt.fisher.mat(pv,times,x.max)
  
  x <- t(cc) %*% solve(S + N*f.I) %*% cc
  return(x)
}

comp.exp.utility <- function(draws,pis,x.low,x.mid,x.max,S,times,p,Deg,accel,N,niter = 10000){
  #This function will calculate the expected utility for compromise test plans
  ###pis is the allocation,x.low, x.mid, and x.max is 
  ##the compromise points for low, middle, and max levels
  #draws, times,p, Deg, S, accel, and N are for the posterior draws, the inspection times,,
  ##quantile, degradation threshold,
  ##prior precision matrix, accelerating level, and sample size
  
  vals <- apply(draws,1,comp.integrand,pis = pis,x.low = x.low,x.mid = x.mid,x.max = x.max,
                times = times,p = p,Deg = Deg,S = S,accel = accel,N = N)
  
  return(mean(vals))
}

o.comp.exp.utility <- function(pis,draws,x.low,x.mid,x.max,S,times,p,Deg,accel,N){
  #This function will calculate the expected utility
  ##used in the R optim function
  ##pi is the allocation,x.low, x.mid, and x.max is 
  ##the compromise points
  #draws, times,p, Deg, S, accel, and N are for the posterior draws, the inspection times,,
  ##quantile, degradation threshold,
  ##prior precision matrix, accelerating level, and sample size

  pi.m = exp(pis[2])/(1+exp(pis[2]))
  pi.l = exp(pis[1])/(1+exp(pis[1]))*(1-pi.m)
  #pi.h = 1-pi.l-pi.m
  
  pis <- c(pi.l,pi.m)
  
  vals <- apply(draws,1,comp.integrand,pis = pis,x.low = x.low,x.mid = x.mid,x.max = x.max,
                times = times,p = p,Deg = Deg,S = S,accel = accel,N = N)
  
  return(mean(vals))
}