
##Set working directories and load libraries and data
library(ggplot2)
resist <- read.csv("resist.txt")
##This code assumes the user has sourced the paper's functions and loaded the data.
##In the following we assume the data is stored in the object "resist"

##Create plots for Figure 1

##Figure 1a
temp.labels =  c(expression(paste('83 ',degree,'C') ), expression(paste('133 ',degree,'C') ), expression(paste('173 ',degree,'C') ) )
ggplot(data=resist,aes(x=hour,y=change,group = unit,linetype = factor(temp))) + geom_point() + geom_line() + 
  xlab('time (hours)') + ylab('relative change') + scale_linetype_discrete('Temperature', labels = temp.labels)

##Figure 1b
ggplot(data=resist,aes(x=hour,y=change,group = unit,linetype = factor(temp))) + geom_point() + geom_line() + 
  scale_x_sqrt('time (hours)',breaks =unique(resist$hour), labels = unique(resist$hour)) + ylab('relative change') + scale_linetype_discrete('Temperature', labels = temp.labels)

##Setting up values needed
##Order of vector is beta mean, beta variance, gamma mean, gamma variance,
##covariance of beta and gamma, log sigma.b mean, log sigma.b sd,
##log sigma mean, and log sigma sd

##Below are the setups for the different cases used

##Case A
hyp.par <- c(0.164,0.0000006, ##Prior mean and variance for beta
             0.00445,5.568e-11, ##Prior mean and variance for gamma
             0.988628*sqrt(0.0000006)*sqrt(5.568e-11), ##Prior covariance for beta and gamma
             -3.86,0.000002, ##Prior mean and standard deviation for log(sigma_b)
             -1.498,0.000006) ##Prior mean and standard deviation for log(sigma)
S <- 0

##Case B
hyp.par <- c(0.164,0.0006, ##Prior mean and variance for beta
             0.00445,5.568e-7, ##Prior mean and variance for gamma
             1.807e-5,  ##Prior covariance for beta and gamma
             -3.86,0.152,  ##Prior mean and standard deviation for log(sigma_b)
             -1.498,0.057) ##Prior mean and standard deviation for log(sigma)
S <- 0

##Case C
hyp.par <- c(0.164,0.0006, ##Prior mean and variance for beta
             0.00445,5.568e-7, ##Prior mean and variance for gamma
             1.807e-5,  ##Prior covariance for beta and gamma
             -3.86,0.152,  ##Prior mean and standard deviation for log(sigma_b)
             -1.498,0.057) ##Prior mean and standard deviation for log(sigma)

sig.b.var = (exp(hyp.par[7]^2)-1)*exp(2*hyp.par[6] + hyp.par[7]^2)
sig.m.var = (exp(hyp.par[9]^2)-1)*exp(2*hyp.par[8] + hyp.par[9]^2)
S = matrix(0,nrow = 4,ncol = 4)
diag(S) <- c(hyp.par[2],hyp.par[4],sig.b.var ,sig.m.var )
S[1,2] <- S[2,1] <- 1.807e-5
S = solve(S)


##Establish other parameters for the design algorithm
Deg <- 1  ##Failure threshold
times = sqrt(unique(resist[,4])) ##Measurement times
p = 0.1 ##For quantile specification
accel = -11605/(50 + 273.15) ##USe conditions converted to Arrhenius scale

##Get draws from the prior for the design algorithm
draws = sample_prior(n=5, hyper.parameters = hyp.par)


##Setting up the design space 
##Convert temperature to Arrhenius scale
xx <- -11605/(seq(83,173,by =5) + 273.15)


##Matrix to store the points in the design space
X.sp <- matrix(xx, nrow = length(xx), ncol = 1)

##Now create the initial starting design
n.init <- 10  ##Number of points in initial design
inits <- runif(n.init, min = 0, max = 1)

##Convert to an index
U <- t(t(ceiling(inits*nrow(X.sp))))


n = 29
##Loop for adding points to design
maxderiv <- 10000
#system.time({
while(maxderiv > 100){
  
  ##Calculate the expected utility for current design
  ##Inputs are the prior draws (draws), the current design (eta),
  ##the times (times), p for the quantile (p), degradation threshold (deg),
  ##accelerating level (accel), prior precision matrix (S), and the sample size (N)
  DD <- exp.utility(draws = draws,eta = X.sp[U,1],times = times,
                    p = p,Deg = Deg,accel = accel, S = S,N=n)
  
  ##Calculate directional derivative at each point in the
  ##design space
  deri <- matrix(0,nrow = nrow(X.sp), ncol = 1)
  thetas <- matrix(0,nrow = nrow(X.sp), ncol = 4)
  for(i in 1:nrow(X.sp)){
    ##Calculate the directional derivative
    ##Inputs are the prior draws (draws), the current singular design (eta.c),
    ##the current design (eta)
    ##the times (times), p for the quantile (p), degradation threshold (deg),
    ##accelerating level (accel), prior precision matrix (S), and the sample size (N)
    t.der <- frechet.der(draws = draws,eta.c = X.sp[i], eta = X.sp[U,1],
                         accel = accel,S = S, times = times, p = p, Deg = Deg,N=n)
    t.id <- which.max(t.der)
    thetas[i,] <- draws[t.id,]
    deri[i,1] <- mean(t.der) + DD 
  }
  
  ##now pick points where the derivative is largest and add to current design
  temp.max <- which.max(deri)
  maxderiv <- deri[temp.max,1]
  U <- c(U,c(temp.max))

  cat("Adding ",-11605/(X.sp[temp.max])-273.15, " with max deriv = ", maxderiv, "\n" )

}
#})

##Viewing and saving the results---------------------------------------------------------
table(U)/length(U)
brks = -11605/(seq(83,173,by =10) + 273.15)

deriv.df = data.frame(x=X.sp, deri = deri, temp = seq(83,173,by =5))

ggplot(data=deriv.df,aes(x=x,y=deri)) + geom_line() + geom_hline(aes(yintercept = 0))+
  scale_x_continuous('temperature (Degrees C)',breaks = brks,labels = seq(83,173,by =10) )+
  ylab(expression(Lambda))



